import { useState, useEffect } from 'react';
import { Plugins } from '@capacitor/core';
import { notAvailable, FeatureNotAvailableError } from './util/models';
import { isFeatureAvailable } from './util/feature-check';
;
;
const availableFeatures = {
    getCurrentPosition: isFeatureAvailable('Geolocation', 'getCurrentPosition'),
    watchPosition: isFeatureAvailable('Geolocation', 'watchPosition')
};
function useCurrentPosition(options) {
    const { Geolocation } = Plugins;
    if (!availableFeatures.getCurrentPosition) {
        return Object.assign({ getPosition: () => { throw new FeatureNotAvailableError(); } }, notAvailable);
    }
    const [currentPosition, setCurrentPosition] = useState();
    const [error, setError] = useState();
    const getPosition = async (newOptions) => {
        let called = false;
        // we use watchPosition here and grab the first result because getCurrentPosition currently has some issues
        const id = Geolocation.watchPosition(newOptions || options || {}, (pos, err) => {
            // watchPosition will sometimes fire updates quickly, 
            // so we check here to make sure its only called one per getPosition invocation
            if (!called) {
                if (err) {
                    setError(err);
                }
                setCurrentPosition(pos);
                // run on next tick so id will be defined
                setTimeout(() => Geolocation.clearWatch({ id }), 0);
                called = true;
            }
        });
    };
    useEffect(() => {
        getPosition(options);
    }, [options]);
    return {
        error,
        currentPosition,
        getPosition,
        isAvailable: true
    };
}
function useWatchPosition() {
    const { Geolocation } = Plugins;
    if (!availableFeatures.watchPosition) {
        return Object.assign({ clearWatch: () => { throw new FeatureNotAvailableError(); }, startWatch: () => { throw new FeatureNotAvailableError(); } }, notAvailable);
    }
    const [currentPosition, setCurrentPosition] = useState();
    const [watchId, setWatchId] = useState('');
    const [error, setError] = useState();
    const clearWatch = () => {
        if (watchId) {
            Geolocation.clearWatch({ id: watchId });
            setWatchId('');
        }
    };
    const startWatch = (options) => {
        if (!watchId) {
            const id = Geolocation.watchPosition(options || {}, (pos, err) => {
                if (err) {
                    setError(err);
                }
                setCurrentPosition(pos);
            });
            setWatchId(id);
        }
    };
    return {
        error,
        currentPosition,
        clearWatch,
        startWatch,
        isAvailable: true
    };
}
export const GeolocationHooks = {
    useCurrentPosition,
    useWatchPosition,
    availableFeatures
};
//# sourceMappingURL=useGeolocation.js.map