/*! Ionic React Hooks: https://ionicframework.com/ - MIT Licensed */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const react = require('react');
const core = require('@capacitor/core');

const allTrue = {
    web: true,
    ios: true,
    android: true,
    electron: true
};
const featureMap = {
    Accessibility: {
        isScreenReaderAvailable: Object.assign(Object.assign({}, allTrue), { web: false }),
        speak: {
            web: 'speechSynthesis' in window,
            ios: true,
            android: true,
            electron: true
        }
    },
    App: {
        state: allTrue,
        getLaunchUrl: Object.assign(Object.assign({}, allTrue), { web: false }),
        appUrlOpen: Object.assign(Object.assign({}, allTrue), { web: false })
    },
    Browser: {
        open: allTrue,
        prefetch: Object.assign(Object.assign({}, allTrue), { web: false }),
        close: Object.assign(Object.assign({}, allTrue), { web: false })
    },
    Camera: {
        getPhoto: allTrue
    },
    Clipboard: {
        useClipboard: Object.assign(Object.assign({}, allTrue), { web: 'clipboard' in navigator }),
    },
    Device: {
        getInfo: allTrue,
        getLanguageCode: allTrue
    },
    Geolocation: {
        getCurrentPosition: Object.assign(Object.assign({}, allTrue), { web: 'geolocation' in navigator }),
        watchPosition: Object.assign(Object.assign({}, allTrue), { web: 'geolocation' in navigator })
    },
    Network: {
        getStatus: allTrue
    },
    Platform: {
        getPlatform: allTrue
    },
    Storage: {
        useStorage: allTrue,
    }
};
function isFeatureAvailable(plugin, method) {
    if (core.Capacitor.isPluginAvailable(plugin) && !!featureMap[plugin][method][core.Capacitor.platform]) {
        return true;
    }
    return false;
}

const notAvailable = {
    isAvailable: false
};
class FeatureNotAvailableError extends Error {
    constructor() {
        super();
        this.message = 'Feature not available on this platform/device. Check availability before attempting to call this method';
        this.name = 'FeatreNotAvailableError';
    }
}

const availableFeatures = {
    isScreenReaderAvailable: isFeatureAvailable('Accessibility', 'isScreenReaderAvailable'),
    speak: isFeatureAvailable('Accessibility', 'speak')
};
function useIsScreenReaderEnabled() {
    const { Accessibility } = core.Plugins;
    if (!availableFeatures.isScreenReaderAvailable) {
        return notAvailable;
    }
    const [isScreenReaderEnabled, setIsScreenReaderAvailable] = react.useState();
    react.useEffect(() => {
        async function checkScreenReader() {
            const isEnabled = await Accessibility.isScreenReaderEnabled();
            setIsScreenReaderAvailable(isEnabled.value);
        }
        if (availableFeatures.isScreenReaderAvailable) {
            checkScreenReader();
        }
    }, [Accessibility, setIsScreenReaderAvailable]);
    return {
        isScreenReaderEnabled,
        isAvailable: true
    };
}
function useSpeak() {
    const { Accessibility } = core.Plugins;
    if (!availableFeatures.speak) {
        return Object.assign({ speak: () => { throw new FeatureNotAvailableError(); } }, notAvailable);
    }
    return {
        speak: Accessibility.speak,
        isAvailable: true
    };
}
const AccessibilityHooks = {
    useIsScreenReaderEnabled,
    useSpeak,
    availableFeatures
};

const availableFeatures$1 = {
    appState: isFeatureAvailable('App', 'state'),
    getLaunchUrl: isFeatureAvailable('App', 'getLaunchUrl'),
    appUrlOpen: isFeatureAvailable('App', 'appUrlOpen')
};
function useAppState() {
    const { App } = core.Plugins;
    if (!availableFeatures$1.appState) {
        return notAvailable;
    }
    const [state, setAppState] = react.useState(true);
    react.useEffect(() => {
        const listener = App.addListener('appStateChange', async (state) => {
            setAppState(state.isActive);
        });
        return () => listener.remove();
    }, [App, setAppState]);
    return {
        state,
        isAvailable: true
    };
}
/**
 * Get the URL the app was originally launched with. Note: if
 * you want to detect future app opens, use `useAppUrlOpen` instead,
 * which will stay updated.
 */
function useLaunchUrl() {
    const { App } = core.Plugins;
    if (!availableFeatures$1.getLaunchUrl) {
        return notAvailable;
    }
    const [launchUrl, setUrl] = react.useState();
    react.useEffect(() => {
        async function getAppLaunchUrl() {
            const ret = await App.getLaunchUrl();
            setUrl(ret.url);
        }
        getAppLaunchUrl();
    }, [App, setUrl]);
    return {
        launchUrl,
        isAvailable: true
    };
}
function useAppUrlOpen() {
    const { App } = core.Plugins;
    if (!isFeatureAvailable('App', 'appUrlOpen')) {
        return notAvailable;
    }
    const [appUrlOpen, setAppUrl] = react.useState();
    react.useEffect(() => {
        const listener = App.addListener('appUrlOpen', async (state) => {
            setAppUrl(state.url);
        });
        return () => listener.remove();
    }, [App, setAppUrl]);
    return {
        appUrlOpen,
        isAvailable: true
    };
}
const AppHooks = {
    useAppState,
    useLaunchUrl,
    useAppUrlOpen,
    availableFeatures: availableFeatures$1
};

const availableFeatures$2 = {
    close: isFeatureAvailable('Browser', 'close'),
    open: isFeatureAvailable('Browser', 'open'),
    prefetch: isFeatureAvailable('Browser', 'prefetch')
};
function useClose() {
    const { Browser } = core.Plugins;
    if (!availableFeatures$2.open) {
        return Object.assign({ close: () => { throw new FeatureNotAvailableError(); } }, notAvailable);
    }
    return {
        close: Browser.close,
        isAvailable: true
    };
}
function useOpen() {
    const { Browser } = core.Plugins;
    if (!availableFeatures$2.open) {
        return Object.assign({ open: () => { throw new FeatureNotAvailableError(); } }, notAvailable);
    }
    return {
        open: Browser.open,
        isAvailable: true
    };
}
function usePrefetch() {
    const { Browser } = core.Plugins;
    if (!availableFeatures$2.prefetch) {
        return Object.assign({ prefetch: () => { throw new FeatureNotAvailableError(); } }, notAvailable);
    }
    return {
        prefetch: Browser.prefetch,
        isAvailable: true
    };
}
const BrowserHooks = {
    useClose,
    useOpen,
    usePrefetch,
    availableFeatures: availableFeatures$2
};

const availableFeatures$3 = {
    getPhoto: isFeatureAvailable('Camera', 'getPhoto')
};
function useCamera() {
    const { Camera } = core.Plugins;
    const [photo, setPhoto] = react.useState();
    if (!availableFeatures$3.getPhoto) {
        return Object.assign({ getPhoto: () => { throw new FeatureNotAvailableError(); } }, notAvailable);
    }
    const getPhoto = async (options) => {
        const photoObject = await Camera.getPhoto(options);
        setPhoto(photoObject);
    };
    return {
        photo,
        getPhoto,
        isAvailable: true
    };
}
const CameraHooks = {
    useCamera,
    availableFeatures: availableFeatures$3
};

const availableFeatures$4 = {
    useClipboard: isFeatureAvailable('Clipboard', 'useClipboard')
};
function useClipboard() {
    const { Clipboard } = core.Plugins;
    if (!(availableFeatures$4.useClipboard)) {
        return Object.assign({ getValue: () => { throw new FeatureNotAvailableError(); }, setValue: () => { throw new FeatureNotAvailableError(); } }, notAvailable);
    }
    const [data, setData] = react.useState();
    async function getValue() {
        const ret = await Clipboard.read({
            type: 'string'
        });
        setData(ret.value);
    }
    async function setValue(value) {
        await Clipboard.write({
            string: value
        });
    }
    return {
        value: data,
        getValue,
        setValue,
        isAvailable: true
    };
}
const ClipboardHooks = {
    useClipboard,
    availableFeatures: availableFeatures$4
};

const availableFeatures$5 = {
    getInfo: isFeatureAvailable('Device', 'getInfo'),
    getLanguageCode: isFeatureAvailable('Device', 'getLanguageCode')
};
function useGetInfo() {
    const { Device } = core.Plugins;
    if (!availableFeatures$5.getInfo) {
        return notAvailable;
    }
    const [info, setInfo] = react.useState();
    react.useEffect(() => {
        async function getInfo() {
            const data = await Device.getInfo();
            setInfo(data);
        }
        getInfo();
    }, [Device, setInfo]);
    return {
        info,
        isAvailable: true
    };
}
function useGetLanguageCode() {
    const { Device } = core.Plugins;
    if (!availableFeatures$5.getLanguageCode) {
        return notAvailable;
    }
    const [languageCode, setLanguageCode] = react.useState();
    react.useEffect(() => {
        async function getLanguageCode() {
            const data = await Device.getLanguageCode();
            setLanguageCode(data.value);
        }
        getLanguageCode();
    }, [Device, setLanguageCode]);
    return {
        languageCode,
        isAvailable: true
    };
}
const DeviceHooks = {
    useGetInfo,
    useGetLanguageCode,
    availableFeatures: availableFeatures$5
};

const availableFeatures$6 = {
    getCurrentPosition: isFeatureAvailable('Geolocation', 'getCurrentPosition'),
    watchPosition: isFeatureAvailable('Geolocation', 'watchPosition')
};
function useCurrentPosition(options) {
    const { Geolocation } = core.Plugins;
    if (!availableFeatures$6.getCurrentPosition) {
        return Object.assign({ getPosition: () => { throw new FeatureNotAvailableError(); } }, notAvailable);
    }
    const [currentPosition, setCurrentPosition] = react.useState();
    const [error, setError] = react.useState();
    const getPosition = async (newOptions) => {
        let called = false;
        // we use watchPosition here and grab the first result because getCurrentPosition currently has some issues
        const id = Geolocation.watchPosition(newOptions || options || {}, (pos, err) => {
            // watchPosition will sometimes fire updates quickly, 
            // so we check here to make sure its only called one per getPosition invocation
            if (!called) {
                if (err) {
                    setError(err);
                }
                setCurrentPosition(pos);
                // run on next tick so id will be defined
                setTimeout(() => Geolocation.clearWatch({ id }), 0);
                called = true;
            }
        });
    };
    react.useEffect(() => {
        getPosition(options);
    }, [options]);
    return {
        error,
        currentPosition,
        getPosition,
        isAvailable: true
    };
}
function useWatchPosition() {
    const { Geolocation } = core.Plugins;
    if (!availableFeatures$6.watchPosition) {
        return Object.assign({ clearWatch: () => { throw new FeatureNotAvailableError(); }, startWatch: () => { throw new FeatureNotAvailableError(); } }, notAvailable);
    }
    const [currentPosition, setCurrentPosition] = react.useState();
    const [watchId, setWatchId] = react.useState('');
    const [error, setError] = react.useState();
    const clearWatch = () => {
        if (watchId) {
            Geolocation.clearWatch({ id: watchId });
            setWatchId('');
        }
    };
    const startWatch = (options) => {
        if (!watchId) {
            const id = Geolocation.watchPosition(options || {}, (pos, err) => {
                if (err) {
                    setError(err);
                }
                setCurrentPosition(pos);
            });
            setWatchId(id);
        }
    };
    return {
        error,
        currentPosition,
        clearWatch,
        startWatch,
        isAvailable: true
    };
}
const GeolocationHooks = {
    useCurrentPosition,
    useWatchPosition,
    availableFeatures: availableFeatures$6
};

function usePlatform() {
    return {
        platform: core.Capacitor.getPlatform(),
        isAvailable: true
    };
}
const PlatformHooks = {
    usePlatform,
    availableFeatures: {
        getPlatform: true
    }
};

const availableFeatures$7 = {
    getStatus: isFeatureAvailable('Network', 'getStatus')
};
function useStatus() {
    const { Network } = core.Plugins;
    if (!availableFeatures$7.getStatus) {
        return notAvailable;
    }
    const [networkStatus, setStatus] = react.useState();
    react.useEffect(() => {
        async function getStatus() {
            const status = await Network.getStatus();
            setStatus(status);
        }
        getStatus();
    }, [Network, setStatus]);
    react.useEffect(() => {
        const listener = Network.addListener('networkStatusChange', (status) => {
            setStatus(status);
        });
        return () => listener.remove();
    }, [Network, setStatus]);
    return {
        networkStatus,
        isAvailable: true
    };
}
const NetworkHooks = {
    useStatus,
    availableFeatures: availableFeatures$7
};

// Inspired by useLocalStorage from https://usehooks.com/useLocalStorage/
const availableFeatures$8 = {
    useStorage: isFeatureAvailable('Storage', 'useStorage')
};
function useStorage() {
    const { Storage } = core.Plugins;
    if (!availableFeatures$8.useStorage) {
        return Object.assign({ get: () => { throw new FeatureNotAvailableError(); }, set: () => { throw new FeatureNotAvailableError(); }, remove: () => { throw new FeatureNotAvailableError(); }, getKeys: () => { throw new FeatureNotAvailableError(); }, clear: () => { throw new FeatureNotAvailableError(); } }, notAvailable);
    }
    async function get(key) {
        const v = await Storage.get({ key });
        if (v) {
            return v.value;
        }
        return null;
    }
    function set(key, value) {
        return Storage.set({ key, value: value });
    }
    function remove(key) {
        return Storage.remove({ key });
    }
    function getKeys() {
        return Storage.keys();
    }
    function clear() {
        return Storage.clear();
    }
    return { get, set, remove, getKeys, clear, isAvailable: true };
}
function useStorageItem(key, initialValue) {
    const { Storage } = core.Plugins;
    if (!availableFeatures$8.useStorage) {
        return [
            undefined,
            () => { throw new FeatureNotAvailableError(); },
            false
        ];
    }
    const [storedValue, setStoredValue] = react.useState();
    react.useEffect(() => {
        async function loadValue() {
            try {
                const result = await Storage.get({ key });
                if (result.value == undefined && initialValue != undefined) {
                    result.value = typeof initialValue === "string" ? initialValue : JSON.stringify(initialValue);
                    setValue(result.value);
                }
                else {
                    setStoredValue(typeof result.value === 'string' ? result.value : JSON.parse(result.value));
                }
            }
            catch (e) {
                return initialValue;
            }
        }
        loadValue();
    }, [Storage, setStoredValue, initialValue, key]);
    const setValue = async (value) => {
        try {
            setStoredValue(value);
            await Storage.set({ key, value: typeof value === "string" ? value : JSON.stringify(value) });
        }
        catch (e) {
            console.error(e);
        }
    };
    return [
        storedValue,
        setValue,
        true
    ];
}
const StorageHooks = {
    useStorageItem,
    useStorage,
    availableFeatures: availableFeatures$8
};

exports.AccessibilityHooks = AccessibilityHooks;
exports.AppHooks = AppHooks;
exports.BrowserHooks = BrowserHooks;
exports.CameraHooks = CameraHooks;
exports.ClipboardHooks = ClipboardHooks;
exports.DeviceHooks = DeviceHooks;
exports.GeolocationHooks = GeolocationHooks;
exports.NetworkHooks = NetworkHooks;
exports.PlatformHooks = PlatformHooks;
exports.StorageHooks = StorageHooks;
//# sourceMappingURL=index.js.map
